/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../../debug/log.h"
#include "../../debug/exdebug.h"

#include "../../exception.h"

#include "../library/library.h"

#include "manager.h"

#include <memory>
#include <string>

namespace core
{
	// get external module
	template<class type> std::shared_ptr<const type> get(const std::string& library)
	{
		// get remote singleton
		auto p = get_remote_singleton<core_arch_interface_manager_t>(library);

		if (__ISNULLPTR(p))
			throw_exception(null_pointer_exception);

		// get interface
		return p->get<type>();
	}

	// get local module
	template<class type> std::shared_ptr<const type> get(void)
	{
		return get<type>(".");
	}

	// set local module
	template<class type> void set(const std::string& library)
	{
		_debug("changing interface \"%s\" to \"%s\"", typeid(type).name(), library.c_str());

		// get interface
		auto ifp = get<type>(library);

		// overwrite global interface
		get_remote_singleton<core_arch_interface_manager_t>(".")->set<type>(ifp);
	}
};

#define REGISTER_INTERFACE(impl_type, interface_type)	core_arch_interface_manager_t::auto_register<interface_type> g_##interface_type(std::make_shared<impl_type>());