/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "auto_update.h"

core_arch_notify_observer_t::~observer(void)
{
	for (auto& p : this->m_list)
		if (p != nullptr)
			p->unregister_observer(this);

	this->m_list.clear();
}

void core_arch_notify_observer_t::observe(core_arch_notify_actor_t*p)
{
	if (p == nullptr)
		return;

	p->register_observer(this);

	{
		std::lock_guard<std::recursive_mutex> lk(this->m_lock);

		this->m_list.push_back(p);
	}
}

void core_arch_notify_observer_t::unobserve(core_arch_notify_actor_t* p)
{
	if (p == nullptr)
		return;

	p->unregister_observer(this);

	{
		std::lock_guard<std::recursive_mutex> lk(this->m_lock);

		this->m_list.remove(p);
	}
}