/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../../debug/safe.h"

#include "../buffer.h"

namespace core::data::compress
{
	class rle8
	{
	public:
		// RLE8 encoding
		static core_data_buffer_t encode(const core_data_buffer_t& buf)
		{
			// create temp buffer
			core_data_buffer_t ret;

			// browse all bytes
			for (size_t i = 0; i < buf.size(); i++)
			{
				// get current byte
				auto curr = *buf.ptr<unsigned char>(i);

				// n will hold number of repetitions
				size_t n = 1;

				// increment n as long as lower than 256 or buffer size
				while (MAKE_UADD(i, n) < buf.size() && n < 255)
				{
					// quit if bytes do no match
					if (curr != *buf.ptr<unsigned char>(MAKE_UADD(i, n)))
						break;

					// otherelse increment n
					MAKE_UINC(n, (size_t)1);
				}

				// append two bytes
				ret.append<unsigned char>((unsigned char)n, false);
				ret.append<unsigned char>((unsigned char)curr, false);

				// skip N bytes
				MAKE_UINC(i, n - 1);
			}

			// return new buffer
			return ret;
		}

		// RLE8 decode
		static core_data_buffer_t decode(const core_data_buffer_t& buf)
		{
			// create temp buffer
			core_data_buffer_t ret;

			// loop through file
			for (size_t i = 0; i < buf.size(); MAKE_UINC(i, (size_t)2))
			{
				// get current bytes
				auto occurence = *buf.ptr<unsigned char>(i);
				auto symbol = *buf.ptr<unsigned char>(MAKE_UADD(i, (size_t)1));

				// add N times
				while (occurence--)
					ret.append(symbol, false);
			}

			// return buffer
			return ret;
		}
	};
};

// shorthands
using core_data_compress_rle8_t = core::data::compress::rle8;