/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../buffer.h"

namespace core::data::encoder
{
	class hexadecimal
	{
	public:
		// convert hexadecimal to buffer
		static core_data_buffer_t decode(const std::string& str, bool big_endian = true)
		{
			core_data_buffer_t buf;

			auto _index = [=](size_t i)
			{
				if (big_endian)
					return str.length() - i - 1;

				return i;
			};

			auto _decode = [](char c)
			{
				if (c >= '0' && c <= '9')
					return (uint8_t)(c - '0');

				if (c >= 'a' && c <= 'f')
					return (uint8_t)(10 + (int)(c - 'a'));

				if (c >= 'A' && c <= 'F')
					return (uint8_t)(10 + (int)(c - 'A'));

				return (uint8_t)0;
			};

			for (size_t i = 0; i < str.length(); i += 2)
			{
				uint8_t c1, c2;

				if (big_endian)
				{
					c1 = _decode(str[_index(i + 1)]);
					c2 = _decode(str[_index(i)]);
				}
				else
				{
					c1 = _decode(str[_index(i)]);
					c2 = _decode(str[_index(i + 1)]);
				}

				uint8_t value = ((c1 & 0xf) << 4) | (c2 & 0xf);

				buf.append<uint8_t>(value);
			}

			if ((str.length() % 2) == 1)
				buf.append<uint8_t>(_decode(str[_index(str.length() - 1)]) << 4);

			return buf;
		}

		// convert buffer to hexadecimal
		static std::string encode(const core_data_buffer_t& buf, bool big_endian = true, int block_space = 0)
		{
			auto _index = [=](size_t i)
			{
				if (big_endian)
					return buf.size() - i - 1;

				return i;
			};

			auto _atohex = [](unsigned char c)
			{
				const char lookup[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

				char szTmp[2] = { lookup[c], '\0' };

				return std::string(szTmp);
			};

			std::string ret;

			auto p = buf.ptr<unsigned char>(0);

			for (size_t i = 0; i < buf.size(); i++)
			{
				auto curr = p[_index(i)];

				ret += _atohex((curr >> 4) & 0xf);
				ret += _atohex(curr & 0xf);

				if (block_space != 0 && ((i + 1) % block_space) == 0 && (i + 1) < buf.size())
					ret += " ";
			}

			return ret;
		}
	};
};

// shorthands
using core_data_encoder_hexadecimal_t = core::data::encoder::hexadecimal;