/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../buffer.h"

#include <string>

namespace core
{
    // 8 bits checksum helper function
    static uint8_t checksum8(const unsigned char* p, size_t size)
    {
        uint8_t sum = 0;

        // add chars
        while (size >= sizeof(uint8_t))
        {
            sum += ~*((uint8_t*)p);
            p += sizeof(uint8_t);
            size -= sizeof(uint8_t);
        }

        return ~sum;
    }

    // 8 bits checksum (storage buffer version)
    static uint8_t checksum8(const core_data_buffer_t& in)
    {
        return checksum8(in.data(), in.size());
    }

    // 8 bits checksum (string version)
    static uint8_t checksum8(const std::string& in)
    {
        return checksum8((const unsigned char*)in.c_str(), in.length());
    }

    // 16 bits checksum helper function
    static uint16_t checksum16(const unsigned char* p, size_t size)
    {
        uint16_t sum = 0;

        // add shorts
        while (size >= sizeof(uint16_t))
        {
            sum += ~*((uint16_t*)p);
            p += sizeof(uint16_t);
            size -= sizeof(uint16_t);
        }

        // then add chars
        while (size >= sizeof(uint8_t))
        {
            sum += ~*((uint8_t*)p);
            p += sizeof(uint8_t);
            size -= sizeof(uint8_t);
        }

        return ~sum;
    }

    // 16 bits checksum (storage buffer version)
    static uint16_t checksum16(const core_data_buffer_t& in)
    {
        return checksum16(in.data(), in.size());
    }

    // 16 bits checksum (string version)
    static uint16_t checksum16(const std::string& in)
    {
        return checksum16((const unsigned char*)in.c_str(), in.length());
    }

    // 32 bits checksum helper function
    static uint32_t checksum32(const unsigned char* p, size_t size)
    {
        uint32_t sum = 0;

        // add longs as long as possible
        while (size >= sizeof(uint32_t))
        {
            sum += ~*((uint32_t*)p);
            p += sizeof(uint32_t);
            size -= sizeof(uint32_t);
        }

        // then add shorts
        while (size >= sizeof(uint16_t))
        {
            sum += ~*((uint16_t*)p);
            p += sizeof(uint16_t);
            size -= sizeof(uint16_t);
        }

        // then add chars
        while (size >= sizeof(uint8_t))
        {
            sum += ~*((uint8_t*)p);
            p += sizeof(uint8_t);
            size -= sizeof(uint8_t);
        }

        return ~sum;
    }

    // 32 bits checksum (storage buffer version)
    static uint32_t checksum32(const core_data_buffer_t& in)
    {
        return checksum32(in.data(), in.size());
    }

    // 32 bits checksum (string version)
    static uint32_t checksum32(const std::string& in)
    {
        return checksum32((const unsigned char*)in.c_str(), in.length());
    }
};