/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../../utils/strings/format.h"
#include "../../utils/strings/fileparts.h"

#include "base.h"

#include <string>
#include <mutex>

namespace core::debug::logger
{
	/* console logger class */
	class console : public base
	{
	public:
		virtual void notify(const event& e) const override
		{
            std::lock_guard<std::mutex> lock(this->m_lock);

            auto filename_nodir = split_file_path(e.get_filename()).file;

            std::string str = "[" + e.get_string_time() + "]\t" + e.get_string_type() + "\t" + filename_nodir + "\t" + e.get_message();
                
#ifdef _WIN32
            str += std::string("\r\n");
#else
            str += std::string("\n");
#endif

            switch (e.get_type())
            {
            case event::type::DBG_MESSAGE:
                fprintf(stderr, "\033[32m");
                break;

            case event::type::DBG_WARNING:
                fprintf(stderr, "\033[33m");
                break;

            case event::type::DBG_ERROR:
                fprintf(stderr, "\033[31m");
                break;

            case event::type::DBG_CRITICAL:
                fprintf(stderr, "\033[1;31m");
                break;

            case event::type::DBG_EXCEPTION:
                fprintf(stderr, "\033[1;33m");
                break;

            case event::type::DBG_EXDEBUG:
                fprintf(stderr, "\033[3;32m");
                break;
            }

            fprintf(stderr, "%s", str.c_str());
            fprintf(stderr, "\033[0m");
		}

	private:
		mutable std::mutex m_lock;
	};
};

// shorthands
using core_debug_logger_console_t = core::debug::logger::console;