/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "manager.h"

#include "console.h"
#include "windbg.h"

#ifdef _WIN32
#include "evemon.h"
#endif

INITIALIZE_SINGLETON(core_debug_logger_manager_t);

core_debug_logger_manager_t::manager(void)
{
    // disable recursion lock
    {
        std::lock_guard<std::recursive_mutex> lk(this->m_lock);

        this->m_recurse_lock = false;
    }

    // add event monitoring in windows mode
#if defined(_WIN32) && defined(USE_EVEMON)
    add<logger::evemon>();
#endif

    // add console debuggers in _DEBUG mode
#ifdef _DEBUG

#ifdef _WIN32

#ifdef _CONSOLE
    add<logger::console>();
#else
    add<logger::windbg>();
#endif  

#else

    add<logger::console>();

#endif

#endif
}