/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../../utils/strings/format.h"
#include "../../utils/strings/fileparts.h"

#include "base.h"

#include <string>
#include <mutex>

#ifdef _WIN32

#include <Windows.h>

namespace core::debug::logger
{
	/* windows debugger logger class */
	class windbg : public base
	{
	public:
		virtual void notify(const event& e) const override
		{
            std::lock_guard<std::mutex> lock(this->m_lock);

            auto filename_nodir = split_file_path(e.get_filename()).file;

			std::string str = "[" + e.get_string_time() + "]\t" + e.get_string_type() + "\t" + filename_nodir + "\t" + e.get_message() + std::string("\r\n");

			OutputDebugStringA(str.c_str());
		}

	private:
		mutable std::mutex m_lock;
	};
};

// shorthands
using core_debug_logger_windbg_t = core::debug::logger::windbg;

#endif