/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <map>

#include "exception.h"

#include "debug/logger/manager.h"

// extended debug
std::map<std::string, std::function<void(void)>> g_exdebug_callbacks;

void core::ex_debug(const std::string& type, std::function<void(void)> callback)
{
    if (g_exdebug_callbacks.find(type) != g_exdebug_callbacks.end())
    {
        g_exdebug_callbacks[type] = callback;

        return;
    }

    g_exdebug_callbacks.emplace(std::make_pair(type, callback));
}

// trigger a breakpoint
void breakpoint(void)
{
    __debugbreak();
}

// send exception to event monitor
void hook_exception(const core_exception_base_t& e, const std::string& type, const char* filename, unsigned int line)
{
    auto p = core::get_instance<core_debug_logger_manager_t>();

    if (p != nullptr)
        p->notify(core_debug_logger_event_type_t::DBG_EXCEPTION, filename, line, e.to_string());

    // check for extended debug
    auto it = g_exdebug_callbacks.find(type);

    if (it != g_exdebug_callbacks.end())
        if (it->second)
           it->second();
}