/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../utils/types/type.h"

#include "ext/string.h"

#include "proxy.h"

namespace core::resources
{
	template<class type> class async_proxy : public proxy<type, true>, public storage::base
	{
	public:
		using proxy<type, true>::proxy;

		virtual std::string get_typename(void) const override
		{
			return decorate_typename("async_proxy<" + get_name_from_type<type>() + ">");
		}

		// push to storage object
		virtual void push(core_resources_filesystem_object_base_t& container) const
		{
			// create temporary object
			if (this->get() != nullptr)
			{
				core_resources_ext_string_t tmp(this->name().c_str());

				auto p = container.create_sub_object(get_typename(), "name");

				if(p != nullptr)
					tmp.push(*p);
			}

			// set typename
			container.set_type_name(get_typename());
		}

		// pop from storage object
		virtual void pop(const core_resources_filesystem_object_base_t& container)
		{
			// check typename
			if (!is_type_name(container.get_type_name(), get_typename()))
				throw_exception(wrong_type_exception, get_typename(), container.get_type_name());

			// check if "name" exists and load it
			auto p = container.get_sub_object(get_typename(), "name");

			if (p != nullptr)
			{
				core_resources_ext_string_t tmp;

				tmp.pop(*p);

				this->load(tmp);
			}
		}
	};
};

// shorthands
template<class type> using core_resources_async_proxy_t = core::resources::async_proxy<type>;