/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../../data/compress/rle0.h"
#include "../../data/compress/rle8.h"
#include "../../data/buffer.h"

#include "../../exception.h"

#include "exception.h"

#include <tuple>

#define FLAG_ENABLE_RLE8			(1 << 0)
#define FLAG_ENABLE_RLE0			(1 << 1)

namespace core
{
	// type of encoding
	enum
	{
		ENCODING_NONE = 0,
		ENCODING_RLE8 = 1,
		ENCODING_RLE0 = 2,
	};

	// encode storage buffer
	static std::tuple<int, core_data_buffer_t> encode(const core_data_buffer_t& buf, unsigned long flags)
	{
		// try all methods
		const size_t num_buffers = 3;

		core_data_buffer_t buffers[num_buffers];

		buffers[core::ENCODING_NONE] = buf;

		for (size_t i = 1; i < num_buffers; i++)
			buffers[i] = buffers[0];

		if ((flags & FLAG_ENABLE_RLE8) != 0)
			buffers[core::ENCODING_RLE8] = core_data_compress_rle8_t::encode(buf);

		if ((flags & FLAG_ENABLE_RLE0) != 0)
			buffers[core::ENCODING_RLE0] = core_data_compress_rle0_t::encode(buf);

		// find best
		size_t min_size = buffers[0].size();
		int encoding = 0;

		for (size_t i = 1; i < num_buffers; i++)
		{
			if (buffers[i].size() < min_size)
			{
				min_size = buffers[i].size();
				encoding = (int)i;
			}
		}

		// return
		return std::tuple<int, core_data_buffer_t>(encoding, buffers[encoding]);
	}
	
	// decode storage buffer
	static core_data_buffer_t decode(const core_data_buffer_t& buf, int encoding)
	{
		switch (encoding)
		{
		case core::ENCODING_NONE:
			return buf;

		case core::ENCODING_RLE8:
			return core_data_compress_rle8_t::decode(buf);

		case core::ENCODING_RLE0:
			return core_data_compress_rle0_t::decode(buf);

		default:
			throw_exception(unknown_encoding_exception, encoding);
		}
	}
};