/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../../exception.h"

// multiple_container_exception exception class
class multiple_container_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Container has multiple items!";
	}
};

// invalid_filesystem_exception exception class
class invalid_filesystem_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Invalid file system!";
	}
};

// wrong_file_type_exception exception class
class wrong_file_type_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Invalid File type!";
	}
};

// wrong_checksum_exception exception class
class wrong_checksum_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Data may have been corrupted. Aborting!";
	}
};

// invalid_buffer_exception exception class
class invalid_buffer_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Buffer is invalid. Aborting!";
	}
};

// unknown_encoding_exception exception class
class unknown_encoding_exception : public core_exception_base_t
{
public:
	unknown_encoding_exception(int encoding)
	{
		this->m_encoding = encoding;
	}

	virtual std::string to_string(void) const override
	{
		char szTmp[64];
		snprintf(szTmp, sizeof(szTmp), "%d", this->m_encoding);

		return std::string("Unknown encoding method #") + std::string(szTmp) + std::string("!");
	}

private:
	int m_encoding;
};

/* bytesize_not_matching_exception exception class */
class bytesize_not_matching_exception : public core_exception_base_t
{
public:
	bytesize_not_matching_exception(const std::string& name, const std::string& type)
	{
		this->m_name = name;
		this->m_type = type;
	}

	virtual std::string to_string(void) const override
	{
		return std::string("Wrong byte size for type \"") + this->m_type + ("\" in variable \"") + this->m_name + ("\" !");
	}

private:
	std::string m_name, m_type;
};