/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "base.h"
#include "reader.h"
#include "writer.h"
#include "object.h"
#include "encode.h"

#include "spc0/header.h"
#include "spc0/reader.h"
#include "spc0/writer.h"
#include "spc0/object.h"

#include <memory>

 // create file system writer
std::shared_ptr<core::resources::filesystem::writer_base> core::resources::filesystem::create_writer(uint32_t type_ident, uint32_t encryption_key, size_t encryption_block_size, unsigned long ulEncodingFlags)
{
	// check ident
	switch (type_ident)
	{
		case core::resources::filesystem::spc0::IDENT_VERSION:
			return std::make_shared<core::resources::filesystem::spc0::writer>(type_ident, encryption_key, encryption_block_size, ulEncodingFlags);
			break;

		default:
			throw_exception(wrong_file_type_exception);
	}
}

// create file system reader
std::shared_ptr<core::resources::filesystem::reader_base> core::resources::filesystem::create_reader(uint32_t type_ident, uint32_t encryption_key, size_t encryption_block_size)
{
	// check ident
	switch (type_ident)
	{
		case core::resources::filesystem::spc0::IDENT_VERSION:
			return std::make_shared<core::resources::filesystem::spc0::reader>(type_ident, encryption_key, encryption_block_size);
			break;

		default:
			throw_exception(wrong_file_type_exception);
	}
}
