/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../../exception.h"

#include <string>

// register_access_exception exception class
class register_access_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Cannot access registry data!";
	}
};

// unknown_resource_exception exception class
class unknown_resource_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Unknown resource!";
	}
};

// not_storage_base_object_exception exception class
class not_storage_base_object_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Object is not an storeable object!";
	}
};

// wrong_type_exception exception class
class wrong_type_exception : public core_exception_base_t
{
public:
	wrong_type_exception(const std::string& found, const std::string& expected)
	{
		this->m_found = found;
		this->m_expected = expected;
	}

	virtual std::string to_string(void) const override
	{
		return "Incorrect typename! \"" + this->m_found + "\" but \"" + this->m_expected + "\" was expected!";
	}

private:
	std::string m_found, m_expected;
};

// unknown_var_exception exception class
class unknown_var_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Unknown variable";
	}
};

/* bigendian_exception exception class */
class bigendian_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Cannot read/write files on a big endian machine!";
	}
};
