/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../../debug/log.h"

#include "../event/flag.h"

#include "../base.h"

#include "base.h"

#include <memory>
#include <functional>

namespace core::threads::event
{
	/* poll class */
	class poll : public core_threads_base_t
	{
	public:
		using callback_t = std::function<void(std::shared_ptr<core_threads_event_base_t>)>;

	public:
		poll(std::shared_ptr<core_threads_event_base_t> p, callback_t callback)
		{
			this->m_job = p;
			this->m_callback = callback;

			start();
		}

		~poll(void)
		{
			_debug("destroying poll");
		}

	private:
		virtual void run(void) override
		{
			std::shared_ptr<core_threads_event_base_t> p = std::atomic_load(&this->m_job);

			if (p == nullptr)
				return;

			if (!this->m_job->wait(0.1))
				return;

			this->m_callback(this->m_job);

			std::atomic_store(&this->m_job, std::shared_ptr<core_threads_event_base_t>(nullptr));
		}

		virtual bool stop_condition(void) const override
		{
			return std::atomic_load(&this->m_job) == nullptr;
		}

	private:
		std::shared_ptr<core_threads_event_base_t> m_job;

		callback_t m_callback;
	};
};

// shorthands
using core_threads_event_poll_t = core::threads::event::poll;