/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../jobs/base.h"
#include "manager.h"

INITIALIZE_SINGLETON(core_threads_pool_manager_t);

// run
void core_threads_pool_manager_t::worker_thread::run(void)
{
	if (!this->m_work_pending.wait(this->m_max_inactivity_time))
	{
		this->m_stop = true;

		return;
	}

	// load job
	std::shared_ptr<core_threads_jobs_base_t> p = std::atomic_load(&this->m_job);

	// skip if no job
	if (__ISNULLPTR(p))
		return;

	// run job
	p->run();

	// set slot free
	std::atomic_store(&this->m_job, std::shared_ptr<core_threads_jobs_base_t>(nullptr));

	// reset event
	this->m_work_pending.reset();
}