/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../utils.h"

#include "../../debug/log.h"
#include "../../data/buffer.h"
#include "../../data/hash/sha2.h"
#include "../../data/encoder/base64.h"

#include "exception.h"
#include "test.h"

#include <cstdlib>
#include <cstring>

namespace core
{
	// expect a string
	static void expect_data(const std::string& label, const core_data_buffer_t& data1, const core_data_buffer_t& data2)
	{
		// check for success
		bool success = data1.equals(data2);

		// display success and exit
		if (success)
		{
			_success("%s\tis ok", label.c_str());

			return;
		}

		// display failure
		auto hash1 = core_data_encoder_base64_t::encode(sha256(data1));
		auto hash2 = core_data_encoder_base64_t::encode(sha256(data2));

		_failure("%s\thash %s is NOT hash %s", label.c_str(), hash1.c_str(), hash2.c_str());

		// throw exception
		throw_exception(test_failed_exception);
	}
};

#define EXPECT_DATA(label, data1, data2)		core::expect_data(label, data1, data2);