/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../utils.h"

#include "../../debug/log.h"

#include "exception.h"
#include "test.h"

#include <cstdlib>
#include <cstring>

namespace core
{
	// expect exception
	static void expect_exception(const std::string& label, std::function<void(void)> func)
	{
		// check for success
		bool success = false;

		try
		{
			if(func)
				func();
		}
		catch (...) {
			success = true;
		}

		// display success and exit
		if (success)
		{
			_success("%s\thas triggered exception", label.c_str());

			return;
		}

		// display failure
		_failure("%s\thas NOT triggered exception", label.c_str());

		// throw exception
		throw_exception(test_failed_exception);
	}

	// expect no exception
	static void expect_noexception(const std::string& label, std::function<void(void)> func)
	{
		// check for success
		bool success = false;

		try
		{
			if (func)
				func();

			success = true;
		}
		catch (...) {}

		// display success and exit
		if (success)
		{
			_success("%s\thas not triggered exception", label.c_str());

			return;
		}

		// display failure
		_failure("%s\thas triggered exception", label.c_str());

		// throw exception
		throw_exception(test_failed_exception);
	}
};

#define EXPECT_EXCEPTION(label, func)		core::expect_exception(label, func);
#define EXPECT_NOEXCEPTION(label, func)		core::expect_noexception(label, func);