/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../utils.h"

#include "../../debug/log.h"

#include "exception.h"
#include "test.h"

#include <cstdlib>
#include <cstring>

namespace core
{
	// expect integer
	static void expect_int(const std::string& label, size_t value1, size_t value2)
	{
		// check for success
		bool success = value1 == value2;

		// display success and exit
		if (success)
		{
			_success("%s\tis ok", label.c_str());

			return;
		}

		// display failure
		_failure("%s\t%zu is NOT %zu", label.c_str(), value1, value2);

		// throw exception
		throw_exception(test_failed_exception);
	}

	// expect integer
	static void expect_int(const std::string& label, time_t value1, time_t value2)
	{
		// check for success
		bool success = value1 == value2;

		// display success and exit
		if (success)
		{
			_success("%s\tis ok", label.c_str());

			return;
		}

		// display failure
		_failure("%s\t%zu is NOT %zu", label.c_str(), value1, value2);

		// throw exception
		throw_exception(test_failed_exception);
	}

	// expect integer
	static void expect_int(const std::string& label, int value1, int value2)
	{
		// check for success
		bool success = value1 == value2;

		// display success and exit
		if (success)
		{
			_success("%s\tis ok", label.c_str());

			return;
		}

		// display failure
		_failure("%s\t%d is NOT %d", label.c_str(), value1, value2);

		// throw exception
		throw_exception(test_failed_exception);
	}
};

#define EXPECT_INT(label, value1, value2)		core::expect_int(label, value1, value2);
#define EXPECT_ZERO(label, value)				core::expect_int(label, value, 0);