/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../utils.h"

#include "../../debug/log.h"

#include "exception.h"
#include "test.h"

#include <cstdlib>
#include <cstring>
#include <cmath>

namespace core
{
	// expect near
	static void expect_near(const std::string& label, const double value, const double target, const double eps)
	{
		// check for success
		bool success = std::fabs(value - target) <= eps;

		// display success and exit
		if (success)
		{
			_success("%s\tis ok", label.c_str());

			return;
		}

		// display failure
		_failure("%s\t%.8le is NOT %.8le within %.8le", label.c_str(), value, target, eps);

		// throw exception
		throw_exception(test_failed_exception);
	}
};

#define EXPECT_NEAR(label, value, target, eps)		core::expect_near(label, value, target, eps);