/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../strings/format.h"

#include "../../arch/instancers/instancer.h"

#include "exception.h"

#include <string>
#include <cstdio>

namespace core::utils::test
{
	class base : public core_arch_instancer_object_base_t<base>
	{
	public:
		virtual void run(void) const = 0;
	};
};

namespace core
{
	static void print_debug(const std::string& message)
	{
		fprintf(stdout, "\033[1;37;46m");
		fprintf(stdout, "[  DEBUG  ]\t%s", message.c_str());
		fprintf(stdout, "\033[0m");

#ifdef _WIN32
		fprintf(stdout, "\r\n");
#else
		fprintf(stdout, "\n");
#endif
	}

	static void print_success(const std::string& message)
	{
		fprintf(stdout, "\033[1;32m");
		fprintf(stdout, "[ SUCCESS ]\t%s", message.c_str());
		fprintf(stdout, "\033[0m");

#ifdef _WIN32
		fprintf(stdout, "\r\n");
#else
		fprintf(stdout, "\n");
#endif

	}

	static void print_failure(const std::string& message)
	{
		fprintf(stdout, "\033[1;31m");
		fprintf(stdout, "[ FAILURE ]\t%s", message.c_str());
		fprintf(stdout, "\033[0m");
		
#ifdef _WIN32
		fprintf(stdout, "\r\n");
#else
		fprintf(stdout, "\n");
#endif
	}

	static void print_comment(const std::string& message)
	{
		fprintf(stdout, "\033[1;33m");
		fprintf(stdout, "[ ------- ]\t%s", message.c_str());
		fprintf(stdout, "\033[0m");

#ifdef _WIN32
		fprintf(stdout, "\r\n");
#else
		fprintf(stdout, "\n");
#endif
	}
};

// shorthands
using core_utils_test_base_t = core::utils::test::base;

#ifdef _MSC_VER

#define _success(szfmt, ...)		                core::print_success(core::format(szfmt, __VA_ARGS__));
#define _failure(szfmt, ...)						core::print_failure(core::format(szfmt, __VA_ARGS__));
#define _comment(szfmt, ...)						core::print_comment(core::format(szfmt, __VA_ARGS__));
#define _verbose(szfmt, ...)						core::print_debug(core::format(szfmt, __VA_ARGS__));

#else

#define _success(szfmt, ...)		                core::print_success(core::format(szfmt __VA_OPT__(,) __VA_ARGS__));
#define _failure(szfmt, ...)						core::print_failure(core::format(szfmt __VA_OPT__(,) __VA_ARGS__));
#define _comment(szfmt, ...)						core::print_comment(core::format(szfmt __VA_OPT__(,) __VA_ARGS__));
#define _verbose(szfmt, ...)						core::print_debug(core::format(szfmt __VA_OPT__(,) __VA_ARGS__));

#endif

#define REGISTER_TEST_EX(class,name)				DECLARE_INSTANCEABLE_OBJECT_EX(class, name, core_utils_test_base_t);
#define REGISTER_TEST(class)						DECLARE_INSTANCEABLE_OBJECT(class, core_utils_test_base_t);

#include "expect_data.h"
#include "expect_string.h"
#include "expect_bool.h"
#include "expect_exception.h"
#include "expect_int.h"
#include "expect_ptr.h"
#include "expect_near.h"