/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#pragma once

#include "../../exception.h"

#include "../utils.h"

/* cannot_resolve_typename_exception exception class */
class cannot_resolve_typename_exception : public core_exception_base_t
{
public:
	cannot_resolve_typename_exception(const std::string& type_name)
	{
		this->m_type_name = type_name;
	}

	virtual std::string to_string(void) const override
	{
		return "Cannot resolve typename of \"" + this->m_type_name + "\" !";
	}

private:
	std::string m_type_name;
};

/* ill_formated_typename_exception exception class */
class ill_formated_typename_exception : public core_exception_base_t
{
public:
	ill_formated_typename_exception(const std::string& type_name)
	{
		this->m_type_name = type_name;
	}

	virtual std::string to_string(void) const override
	{
		return "Ill formated typename \"" + this->m_type_name + "\" !";
	}

private:
	std::string m_type_name;
};

/* cannot_resolve_typename_exception exception class */
class unknown_typename_exception : public core_exception_base_t
{
public:
	unknown_typename_exception(const std::string& type_name)
	{
		this->m_type_name = type_name;
	}

	virtual std::string to_string(void) const override
	{
		return "Unknown typename \"" + this->m_type_name + "\" !";
	}

private:
	std::string m_type_name;
};

/* wrong_size_exception exception class */
class wrong_size_exception : public core_exception_base_t
{
public:
	wrong_size_exception(const size_t size1, const size_t size2)
	{
		this->m_size1 = size1;
		this->m_size2 = size2;
	}

	virtual std::string to_string(void) const override
	{
		char szTmp1[64], szTmp2[64];

		snprintf(szTmp1, sizeof(szTmp1), "%zu", this->m_size1);
		snprintf(szTmp2, sizeof(szTmp2), "%zu", this->m_size2);

		return "Wrong size " + std::string(szTmp1) + std::string(" when type size is ") + std::string(szTmp2) + std::string(" bytes!");
	}

private:
	size_t m_size1, m_size2;
};
