/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/arch/instancers/instancer.h"
#include "../../../common/arch/instancers/local_instancer.h"
#include "../../../common/utils/test/test.h"

#include "../../test-common/common.h"

/* specialized global class */
class core_arch_instancer_specialized_mockup : public core_arch_instancer_base_mockup
{
public:
	virtual int get(void) const override
	{
		return 1;
	}
};

DECLARE_INSTANCEABLE_OBJECT(core_arch_instancer_specialized_mockup, core_arch_instancer_base_mockup);

/* specialized local class */
class core_arch_instancer_specialized_local_mockup : public core_arch_instancer_base_mockup
{
public:
	virtual int get(void) const override
	{
		return 2;
	}
};

REGISTER_TYPENAME(core_arch_instancer_specialized_local_mockup);

/* instancers tests */
class core_arch_instancers_test : public core_utils_test_base_t
{
private:
	class subclass : public core_arch_instancer_local_t<core_arch_instancer_base_mockup>
	{
	public:
		subclass(void)
		{
			DECLARE_LOCAL_INSTANCER(core_arch_instancer_specialized_local_mockup);
		}

		void test(void)
		{
			// local object
			{
				auto p = this->create_instance("core_arch_instancer_specialized_local_mockup");

				EXPECT_NOTNULL("instance of \"core_arch_instancer_specialized_local_mockup\" shall not be null", p.get());
				EXPECT_INT("object content shall be 2", p->get(), 2);
			}

			// instanciate unknown object
			{
				EXPECT_EXCEPTION("instancing unknown object shall trigger exception", [&](void)
					{
						auto p = this->create_instance("core_arch_instancer_unknown_mockup");
					});
			}
		}
	};

public:
	virtual void run(void) const override
	{
		_comment("Instancers tests");

		// instanciate a global object
		{
			auto p = core::create_instance<core_arch_instancer_base_mockup>("core_arch_instancer_specialized_mockup");

			EXPECT_NOTNULL("instance of \"core_arch_instancer_specialized_mockup\" shall not be null", p.get());
			EXPECT_INT("object content shall be 1", p->get(), 1);
		}

		// instanciate unknown object
		{
			EXPECT_EXCEPTION("instancing unknown object shall trigger exception", [](void)
				{
					auto p = core::create_instance<core_arch_instancer_base_mockup>("core_arch_instancer_unknown_mockup");
				});
		}

		// instanciate local object
		{
			subclass obj;

			obj.test();
		}

		// instanciate remote library object
		{
			auto p = core::create_instance<core_arch_instancer_base_mockup>("core-test-lib/core_arch_instancer_specialized_remote_mockup");

			EXPECT_NOTNULL("instance of remote \"core-test-lib/core_arch_instancer_specialized_remote_mockup\" shall not be null", p.get());
			EXPECT_INT("object content shall be 3", p->get(), 3);
		}
	}
};

REGISTER_TEST(core_arch_instancers_test);