/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/arch/library/manager.h"
#include "../../../common/arch/interface/interface.h"
#include "../../../common/arch/library/library.h"
#include "../../../common/utils/test/test.h"

#include "../../test-common/common.h"

/* library tests */
class core_arch_library_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Library tests");

		// check library existances
		EXPECT_TRUE("shall have a library called \"core-test-lib\"", core::get_remote_singleton<core_arch_library_manager_t>(".")->has("core-test-lib"));

		{
			bool found = false;

			for (auto& v : core::get_remote_singleton<core_arch_library_manager_t>(".")->list())
				if (v == "core-test-lib")
					found = true;

			EXPECT_TRUE("libraries manager list shall contain entry \"core-test-lib\"", found);
		}

		// call specific function
		{
			// load function
			auto func = (int(*)(void))core::get_remote_singleton<core_arch_library_manager_t>(".")->get_addr("core-test-lib", "core_arch_library_func");

			EXPECT_NOTNULL("function pointer shall not be null", (void*)func);
			EXPECT_INT("function result shall be 1", (*func)(), 1);

			// shall trigger exception
			EXPECT_EXCEPTION("unknown function shall trigger exception", [](void)
				{
					auto func = core::get_remote_singleton<core_arch_library_manager_t>(".")->get_addr("core-test-lib", "core_arch_library_unknown_func");
				});
		}

		// create sample interface
		{
			// current interface shall be null
			EXPECT_EXCEPTION("default library shall be null", [](void)
				{
					auto p = core::get<core_arch_interface_mockup>();
				});

			// set current interface
			core::set<core_arch_interface_mockup>("core-test-lib");

			// get value
			auto value = core::get<core_arch_interface_mockup>()->test();

			EXPECT_INT("remote interface test function shall return 1", value, 1);

			// shall trigger exception
			EXPECT_EXCEPTION("unknown library shall trigger exception", [](void)
				{
					core::set<core_arch_interface_mockup>("core-unknown-lib");
				})
		}

		// test remote
		{
			EXPECT_TRUE("remote library shall have a library_interface_manager_t singleton", core::has_remote_singleton<core_arch_interface_manager_t>("core-test-lib"));
		}
	}
};

REGISTER_TEST(core_arch_library_test);