/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/arch/scope/base.h"
#include "../../../common/arch/scope/access.h"
#include "../../../common/utils/test/test.h"

/* access mockup class */
class core_arch_scope_access_mockup : public core_arch_scope_base_t
{
public:
	class handle : public core_arch_scope_access_t<core_arch_scope_access_mockup>
	{
	public:
		handle(core_arch_scope_access_mockup* parent, const core_arch_scope_ticket_t& handle, const int value) : access(parent, handle)
		{
			this->m_value = value;
		}

		int test(void)
		{
			return get()->test(this->m_value);
		}

	private:
		int m_value;
	};

public:
	auto create(const int value)
	{
		return std::make_unique<handle>(this, get_scope_ticket(), value);
	}

private:
	int test(int value)
	{
		return 2 * value + 1;
	}
};

/* scope tests */
class core_arch_scope_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Scope tests");

		// scope ticket
		{
			{
				core_arch_scope_access_mockup obj;

				auto ticket = obj.get_scope_ticket().create_handle();

				EXPECT_TRUE("ticket shall be active", ticket.is_alive());
			}
		}

		// access
		{
			core_arch_scope_access_mockup obj;

			auto handle = obj.create(1);

			EXPECT_INT("handle test function shall return 3", handle->test(), 3);
		}
	}
};

REGISTER_TEST(core_arch_scope_test);