/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/arch/singleton.h"
#include "../../../common/utils/test/test.h"

/* singleton mock up */
class core_arch_singleton_mockup : public core_arch_singleton_t<core_arch_singleton_mockup>
{
	friend class singleton<core_arch_singleton_mockup>;

public:
	void reset(void)
	{
		this->m_test = 0;
	}

	int test(void)
	{
		return ++this->m_test;
	}

private:
	core_arch_singleton_mockup(void)
	{
		reset();
	}

private:

	int m_test;
};

INITIALIZE_SINGLETON(core_arch_singleton_mockup);

/* singleton tests */
class core_arch_singleton_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Singleton tests");

		// reset singleton (in case of)
		core::get_instance<core_arch_singleton_mockup>()->reset();

		// object shall not be recreated
		EXPECT_INT("first call shall give 1", core::get_instance<core_arch_singleton_mockup>()->test(), 1);
		EXPECT_INT("second call shall give 2", core::get_instance<core_arch_singleton_mockup>()->test(), 2);
	}
};

REGISTER_TEST(core_arch_singleton_test);