/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <stdio.h>

#include "../../../common/utils/utils.h"
#include "../../../common/debug/log.h"
#include "../../../common/exception.h"
#include "../../../common/utils/test/test.h"
#include "../../../common/debug/logger/hook.h"

/* file Test */
class core_debug_file_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Event Monitoring Content Test");

		// file, line & content
		{
			auto p = core_debug_logger_hook_t();

			_debug("Hello World!");

			EXPECT_TRUE("message shall have correct file name, line number and content", p->has({ }, { std::regex(".*core_debug_content_test.cpp:29$") }, { std::regex("Hello World!") }));
		}

		// filter by type
		{
			auto p = core_debug_logger_hook_t();

			core::get_instance<core_debug_logger_manager_t>()->filter({ core_debug_logger_event_type_t::DBG_MESSAGE }, {}, {});

			_debug("Hello World!");
			_warning("Hello World!");

			EXPECT_INT("event monitoring can be filtered by type", (int)p->find({ }, { std::regex(".*core_debug_content_test.cpp:[0-9]+$") }, { }).size(), 1);
		}

		// filter by message
		{
			auto p = core_debug_logger_hook_t();

			core::get_instance<core_debug_logger_manager_t>()->filter({ }, {}, { std::regex("Hello World!") });

			_debug("Hello World!");
			_debug("Some other text!");

			EXPECT_INT("event monitoring can be filtered by message", (int)p->find({ }, { std::regex(".*core_debug_content_test.cpp:[0-9]+$") }, { }).size(), 1);
		}

		// filter by filename
		{
			auto p = core_debug_logger_hook_t();

			core::get_instance<core_debug_logger_manager_t>()->filter({ }, { std::regex(".*core_debug_content_test.cpp:65$") }, { });

			_debug("Hello World!");
			_debug("Some other text!");

			EXPECT_INT("event monitoring can be filtered by filename & line number", (int)p->find({ }, { std::regex(".*core_debug_content_test.cpp:[0-9]+$") }, { }).size(), 1);
		}

		// restore filters
		core::get_instance<core_debug_logger_manager_t>()->filter({ }, { }, { });
	}
};

REGISTER_TEST(core_debug_file_test);