/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <stdio.h>

#include "../../../common/utils/utils.h"
#include "../../../common/debug/log.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/utils/test/test.h"
#include "../../../common/debug/logger/hook.h"

/* Extended Debug Test */
class core_debug_exdebug_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Event Monitoring Extended Debug Test");

		// nullptr
		{
			int tmp;
			auto p = core_debug_logger_hook_t();

			EXPECT_TRUE("__ISNULLPTR shall be true on null pointers", __ISNULLPTR(nullptr));
			EXPECT_FALSE("__ISNULLPTR shall be false on valid pointers", __ISNULLPTR(&tmp));

#ifdef _EXDEBUG
			EXPECT_INT("__ISNULLPTR shall trigger DBG_EXDEBUG message on null pointers", (int)p->find({ core_debug_logger_event_type_t::DBG_EXDEBUG }, { std::regex(".*exdebug_test.cpp:[0-9]+$") }, { }).size(), 1);
#else
			EXPECT_INT("__ISNULLPTR shall not trigger DBG_EXDEBUG message on null pointers when extended debug is off", (int)p->find({ core_debug_logger_event_type_t::DBG_EXDEBUG }, { std::regex(".*exdebug_test.cpp:[0-9]+$") }, { }).size(), 0);
#endif
		}

		// zero
		{
			auto p = core_debug_logger_hook_t();

			EXPECT_TRUE("__ISZERO shall be true on zeroes", __ISZERO(0));
			EXPECT_FALSE("__ISZERO shall be false on non-zeroes", __ISZERO(1));

#ifdef _EXDEBUG
			EXPECT_INT("__ISZERO shall trigger DBG_EXDEBUG message on zeroes", (int)p->find({ core_debug_logger_event_type_t::DBG_EXDEBUG }, { std::regex(".*exdebug_test.cpp:[0-9]+$") }, { }).size(), 1);
#else
			EXPECT_INT("__ISZERO shall not trigger DBG_EXDEBUG message on zeroes when extended debug is off", (int)p->find({ core_debug_logger_event_type_t::DBG_EXDEBUG }, { std::regex(".*exdebug_test.cpp:[0-9]+$") }, { }).size(), 0);
#endif
		}
	}
};

REGISTER_TEST(core_debug_exdebug_test);