/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <stdio.h>

#include "../../../common/utils/utils.h"
#include "../../../common/debug/log.h"
#include "../../../common/exception.h"
#include "../../../common/utils/test/test.h"
#include "../../../common/debug/logger/hook.h"

/* windows debugger logger class */
class core_debug_recursive_logger : public core_debug_logger_base_t
{
public:
	virtual void notify(const core_debug_logger_event_t& e) const override
	{
		_debug("shall not print!");
	}

private:
};

/* Recursive Test */
class core_debug_recursive_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Event Monitoring Recursive Test");

		// file, line & content
		{
			auto p1 = core::debug::logger::auto_logger<core_debug_recursive_logger>();

			auto p2 = core_debug_logger_hook_t();

			_debug("shall print");

			EXPECT_INT("recursive logging shall not occur", (int)p2->find({ }, { std::regex(".*recursive_test.cpp:[0-9]+$") }, { }).size(), 1);
		}
	}
};

REGISTER_TEST(core_debug_recursive_test);