/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <stdio.h>

#include "../../../common/utils/utils.h"
#include "../../../common/debug/log.h"
#include "../../../common/exception.h"
#include "../../../common/utils/test/test.h"
#include "../../../common/debug/logger/hook.h"

/* hello_world_exception exception class */
class core_debug_hello_world_exception : public core_exception_base_t
{
public:
	virtual std::string to_string(void) const override
	{
		return "Hello World!";
	}
};

/* Types Test */
class core_debug_types_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Event Monitoring Types Test");

		// _debug
		{
			auto p = core_debug_logger_hook_t();

			_debug("Hello World!");

			EXPECT_TRUE("_debug shall trigger a DBG_MESSAGE message", p->has({ core_debug_logger_event_type_t::DBG_MESSAGE }, {}, { std::regex("Hello World!") }));
		}

		// _warning
		{
			auto p = core_debug_logger_hook_t();

			_warning("Hello World!");

			EXPECT_TRUE("_warning shall trigger a DBG_WARNING message", p->has({ core_debug_logger_event_type_t::DBG_WARNING }, {}, { std::regex("Hello World!") }));
		}

		// _error
		{
			auto p = core_debug_logger_hook_t();

			_error("Hello World!");

			EXPECT_TRUE("_error shall trigger a DBG_ERROR message", p->has({ core_debug_logger_event_type_t::DBG_ERROR }, {}, { std::regex("Hello World!") }));
		}

		// _critical
		{
			auto p = core_debug_logger_hook_t();

			_critical("Hello World!");

			EXPECT_TRUE("_critical shall trigger a DBG_CRITICAL message", p->has({ core_debug_logger_event_type_t::DBG_CRITICAL }, {}, { std::regex("Hello World!") }));
		}

		// exception
		{
			auto p = core_debug_logger_hook_t();

			try
			{
				throw_exception(core_debug_hello_world_exception);
			}
			catch (...) {}

			EXPECT_TRUE("throwing exception trigger a DBG_EXCEPTION message", p->has({ core_debug_logger_event_type_t::DBG_EXCEPTION }, {}, { std::regex("Hello World!") }));
		}
	}
};

REGISTER_TEST(core_debug_types_test);