/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/data/encoder/base64.h"
#include "../../../common/utils/test/test.h"

/* base64 encoding against standard test vectors */
class core_encode_base64_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("BASE64 encoding/decoding against RFC4648 test vectors");

		test("", "");
		test("f", "Zg==");
		test("fo", "Zm8=");
		test("foo", "Zm9v");
		test("foob", "Zm9vYg==");
		test("fooba", "Zm9vYmE=");
		test("foobar", "Zm9vYmFy");

		EXPECT_EXCEPTION("unknown character shall trigger an exception", [](void)
			{
				core_data_encoder_base64_t::decode("Zm9vYmF!");
			});

		EXPECT_EXCEPTION("ill-formed message shall trigger an exception", [](void)
			{
				core_data_encoder_base64_t::decode("Zm9vYmF");
			});

		EXPECT_EXCEPTION("ill-formed message shall trigger an exception", [](void)
			{
				core_data_encoder_base64_t::decode("Zm9vYg=A");
			});
	}

private:
	void test(const std::string& str_in, const std::string& enc_expected) const
	{
		core_data_buffer_t in((const unsigned char*)str_in.c_str(), str_in.length());

		auto encoded_string = core_data_encoder_base64_t::encode(in);
		auto decoded_buffer = core_data_encoder_base64_t::decode(encoded_string);

		EXPECT_STR(core::format("BASE64(\"%s\") = \"%s\"", str_in.c_str(), enc_expected.c_str()), encoded_string, enc_expected);
		EXPECT_TRUE(core::format("decoding string \"%s\" gives original vector", enc_expected.c_str()), decoded_buffer.equals(in));
	}
};

REGISTER_TEST(core_encode_base64_test);