/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/data/hash/checksum.h"
#include "../../../common/utils/test/test.h"

/* checksum helper tests (unofficial) */
class core_encode_checksum_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("checksums tests");

		core_data_buffer_t in;

		in.append<uint32_t>(0xdeadbeef);
		in.append<uint16_t>(0xfeed);
		in.append<uint8_t>(0x3f);

		EXPECT_TRUE("CHECKSUM8(\"\") = 0xff", core::checksum8("") == 0xff);
		EXPECT_TRUE("CHECKSUM8(\"ABCDEFGH\") = 0x2b", core::checksum8("ABCDEFGH") == 0x2b);
		EXPECT_TRUE("CHECKSUM8 with storage buffer", core::checksum8(in) == 0x68);

		EXPECT_TRUE("CHECKSUM16(\"\") = 0xffff", core::checksum16("") == 0xffff);
		EXPECT_TRUE("CHECKSUM16(\"ABCDEFGH\") = 0x1513", core::checksum16("ABCDEFGH") == 0x1513);
		EXPECT_TRUE("CHECKSUM16 with storage buffer", core::checksum16(in) == 0x9CCB);

		EXPECT_TRUE("CHECKSUM32(\"\") = 0xffffffff", core::checksum32("") == 0xffffffff);
		EXPECT_TRUE("CHECKSUM32(\"ABCDEFGH\") = 0x8c8a8887", core::checksum32("ABCDEFGH") == 0x8c8a8887);
		EXPECT_TRUE("CHECKSUM32 with storage buffer", core::checksum32(in) == 0xDEAEBE1D);
	}
};

REGISTER_TEST(core_encode_checksum_test);