/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/data/encoder/hexadecimal.h"
#include "../../../common/utils/test/test.h"

/* hexadecimal encoding */
class core_encode_hexadecimal_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("hexadecimal encoding/decoding");

		{
			uint32_t value = 0xfeedbeef;

			core_data_buffer_t in;
			in.append(value);

			// little endian
			{
				auto str = core_data_encoder_hexadecimal_t::encode(in, false);

				EXPECT_STR("encoded value expected to be 0xfeedbeef in little endian", str, "efbeedfe");

				auto decoded = core_data_encoder_hexadecimal_t::decode(str, false);

				EXPECT_TRUE("decoded value expected to be equal to original", *decoded.ptr<uint32_t>(0) == value);
			}

			// big endian
			{
				auto str = core_data_encoder_hexadecimal_t::encode(in, true);

				EXPECT_STR("encoded value expected to be 0xfeedbeef in big endian", str, "feedbeef");

				auto decoded = core_data_encoder_hexadecimal_t::decode(str, true);

				EXPECT_TRUE("decoded value expected to be equal to original", *decoded.ptr<uint32_t>(0) == value);
			}
		}

		// capital decoding
		{
			std::string str = "FEEDBEEF";

			auto decoded = core_data_encoder_hexadecimal_t::decode(str, true);

			EXPECT_TRUE("decoded value expected to be equal to original", *decoded.ptr<uint32_t>(0) == 0xfeedbeef);
		}

		// uneven decoding
		{
			std::string str = "feedbee";

			auto decoded = core_data_encoder_hexadecimal_t::decode(str, true);

			EXPECT_TRUE("decoded value expected to be equal to original", *decoded.ptr<uint32_t>(0) == 0xfeedbee);
		}

		// with block spacing
		{
			core_data_buffer_t in;

			in.append<uint32_t>(0xfeedbeef);
			in.append<uint32_t>(0xdeadbeef);

			auto str = core_data_encoder_hexadecimal_t::encode(in, true, 4);

			EXPECT_STR("encoded value using block spacing expected to be \"deadbeef feedbeef\"", str, "deadbeef feedbeef");
		}
	}
};

REGISTER_TEST(core_encode_hexadecimal_test);