/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/data/compress/rle8.h"
#include "../../../common/utils/test/test.h"

/* RLE8 non-official implementation */
class core_encode_rle8_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("RLE8 non-official implementation");

		test("");
		test("ABCDEFGH");
		test("AAABBBCC");
		test("AAABBBAA");
		test("AAABCDEF");
	}

private:
	void test(const std::string& str_in) const
	{
		core_data_buffer_t in((const unsigned char*)str_in.c_str(), str_in.length());

		auto encoded_buffer = core_data_compress_rle8_t::encode(in);
		auto decoded_buffer = core_data_compress_rle8_t::decode(encoded_buffer);

		EXPECT_TRUE(core::format("decoding buffer of \"%s\" gives original vector", str_in.c_str()), decoded_buffer.equals(in));
	}
};

REGISTER_TEST(core_encode_rle8_test);