/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/arch/notify/auto_update.h"
#include "../../../common/utils/test/test.h"

/* actor mockup */
class core_events_autoupdate_actor_mockup : public core_arch_notify_actor_t { };

/* observer mockup */
class core_events_autoupdate_observer_mockup : public core_arch_notify_observer_t
{
public:
	core_events_autoupdate_observer_mockup(core_events_autoupdate_actor_mockup& actor)
	{
		this->m_flag = false;

		this->observe(&actor);
	}

	bool get(void) const
	{
		return this->m_flag;
	}

protected:
	virtual bool on_update(core_arch_notify_actor_t&) override
	{
		this->m_flag = true;

		return true;
	}

private:
	bool m_flag;
};


/* auto-update test */
class core_events_autoupdate_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Events auto-update testing");

		// simple case
		{
			core_events_autoupdate_actor_mockup actor;

			EXPECT_INT("number of observees shall be 0", actor.num_observees(), (size_t)0);

			{
				core_events_autoupdate_observer_mockup observer(actor);

				EXPECT_INT("number of observees shall be 1 after add", actor.num_observees(), (size_t)1);

				EXPECT_FALSE("observer state shall be set to false", observer.get());

				EXPECT_INT("actor shall have sent 1 signal in total", actor.signal(), (size_t)1);

				EXPECT_TRUE("observer state shall be set to true after signal", observer.get());
			}

			EXPECT_INT("actor shall have sent 0 signal in total after observer destruction", actor.signal(), (size_t)0);
		}

		// destroying actor shall call unobserve
		{
			auto actor = std::make_shared<core_events_autoupdate_actor_mockup>();

			EXPECT_NOTNULL("actor shall not be null", actor.get());

			core_events_autoupdate_observer_mockup observer(*actor);

			EXPECT_INT("number of observed actor shall be 1", observer.num_observed(), (size_t)1);

			actor.reset();

			EXPECT_INT("number of observed actor shall be 0 after pointer reset", observer.num_observed(), (size_t)0);
		}
	}
};

REGISTER_TEST(core_events_autoupdate_test);