/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/threads/event/base.h"
#include "../../../common/threads/event/flag.h"
#include "../../../common/utils/test/test.h"

/* event test */
class core_events_event_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Events testing");

		// event mockup
		{
			core_threads_event_flag_t e;

			EXPECT_TRUE("initial event status shall be pending", e.is_pending());

			e.trigger();

			EXPECT_TRUE("event wait shall not timeout", e.wait(0.1));
			EXPECT_FALSE("event shall not be pending anymore after trigger", e.is_pending());

			e.reset();

			EXPECT_TRUE("event shall be pending again after reset", e.is_pending());

			int test = 0;

			e.do_until([&](void)
				{
					test++;
					e.trigger();
				}, 0.1);

			EXPECT_INT("do_until shall be called exactly once", test, 1);

			e.reset();

			EXPECT_FALSE("untriggered do_until shall fail after timeout", e.do_until([&](void) {}, 0.01));
		}
	}
};

REGISTER_TEST(core_events_event_test);