/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/threads/event/base.h"
#include "../../../common/threads/event/poll.h"
#include "../../../common/threads/event/flag.h"
#include "../../../common/utils/test/test.h"

/* poll test */
class core_events_poll_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Poll testing");

		{
			// event to watch
			auto e = std::make_shared<core_threads_event_flag_t>();

			EXPECT_NOTNULL("event shall not be null", e.get());

			// poll object
			int value = 0;

			core_threads_event_poll_t obj(e, [&](std::shared_ptr<core_threads_event_base_t>)
				{
					value = 1;
				});

			EXPECT_INT("initial value shall be 0", value, 0);

			e->trigger();

			EXPECT_TRUE("poll object shall not timeout", obj.wait(0.1));
			EXPECT_INT("value after trigger shall be 1", value, 1);
		}
	}
};

REGISTER_TEST(core_events_poll_test);