/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <stdio.h>

#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/utils/test/test.h"
#include "../../../common/resources/async_proxy.h"

#include "core_resources_intval.h"

extern int g_num_loads;

/* Async Test */
class core_resources_async_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Resources Async Loading Test");

		const char* resname = "string://10";

		{
			core_resources_async_proxy_t<core_resources_intval> r(resname);

			EXPECT_TRUE("Resource shall be pending", r.is_pending());
			EXPECT_TRUE("Resource loading shall not timeout", r.wait(1.0));
			EXPECT_NOTNULL("Resource shall not be null", r.get());
		}

		{
			// clear counter
			g_num_loads = 0;

			// load resource twice
			core_resources_async_proxy_t<core_resources_intval> r1(resname);
			core_resources_async_proxy_t<core_resources_intval> r2(resname);

			// check that it was only constructed once
			EXPECT_TRUE("Async resources shall be pending", r1.is_pending() && r2.is_pending());
			EXPECT_TRUE("Async resources loading shall not timeout", r1.wait(1.0) && r2.wait(1.0));
			EXPECT_INT("Async resource on same scope shall be constructed only once", g_num_loads, 1);
		}
	}
};

REGISTER_TEST(core_resources_async_test);