/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <stdio.h>

#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/utils/test/test.h"
#include "../../../common/resources/proxy.h"

#include "core_resources_intval.h"

/* Double Load Test */
class core_resources_double_load_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Resources Double Load Test");

		const char* resname = "string://10";

		// same scope
		{
			// clear counter
			g_num_loads = 0;

			// load resource twice
			core_resources_proxy_t<core_resources_intval> r1(resname);
			core_resources_proxy_t<core_resources_intval> r2(resname);

			// check that it was only constructed once
			EXPECT_NOTNULL("Resource 1 shall not be null", r1.get());
			EXPECT_NOTNULL("Resource 2 shall not be null", r2.get());
			EXPECT_INT("Resource on same scope shall be constructed only once", g_num_loads, 1);
		}

		// different scope
		{
			// clear counter
			g_num_loads = 0;

			// load resource twice
			{
				core_resources_proxy_t<core_resources_intval> r(resname);

				EXPECT_NOTNULL("Resource 1 shall not be null", r.get());
			}

			{
				core_resources_proxy_t<core_resources_intval> r(resname);

				EXPECT_NOTNULL("Resource 2 shall not be null", r.get());
			}

			// check that it was constructed twice
			EXPECT_INT("Resource on different scope shall be constructed each time", g_num_loads, 2);
		}
	}
};

REGISTER_TEST(core_resources_double_load_test);