/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <stdio.h>

#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/utils/test/test.h"
#include "../../../common/resources/async_proxy.h"
#include "../../../common/resources/proxy.h"

#include "core_resources_intval.h"

/* Exception Test */
class core_resources_exception_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Resources Exception Test");

		const char* resname = "file://invalid";

		// synchronous shall trigger exception
		EXPECT_EXCEPTION("Invalid synchronous resource shall trigger exception", [&](void)
			{
				core_resources_proxy_t<core_resources_intval> r(resname);
			});

		// asynchronous shall trigger exception
		EXPECT_NOEXCEPTION("Invalid asynchronous resource shall trigger exception", [&](void)
			{
				core_resources_async_proxy_t<core_resources_intval> r(resname);
			});

		// invalid async resource shall be null
		{
			core_resources_async_proxy_t<core_resources_intval> r(resname);

			EXPECT_NULL("Invalid asynchronous resource shall be null", r.get());
		}

		// uninitialized resource
		{
			core_resources_proxy_t<core_resources_intval> r;

			EXPECT_EXCEPTION("Uninitialized synchronous resource shall trigger exception", [&](void)
				{
					r.get();
				});
		}
	}
};

REGISTER_TEST(core_resources_exception_test);