/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <stdio.h>

#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/utils/test/test.h"
#include "../../../common/resources/manager.h"

#include "core_resources_intval.h"

/* Null Resource Test */
class core_resources_null_resource_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Null Resource Test");

		const char* resname = "file://invalid";

		// set null resource
		core::set_null_resource(std::make_shared<const core_resources_intval>(-1));

		// create resource
		core_resources_async_proxy_t<core_resources_intval> r(resname);

		// expect -1
		EXPECT_TRUE("Invalid asynchronous resource shall not timeout", r.wait(1.0));
		EXPECT_NOTNULL("Invalid asynchronous resource shall not be null", r.get());
		EXPECT_INT("Invalid asynchronous resource shall give -1", r->get(), -1);

		// clear null resource
		core::set_null_resource<core_resources_intval>(nullptr);
	}
};

REGISTER_TEST(core_resources_null_resource_test);