/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include <stdio.h>

#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/resources/storage/base.h"
#include "../../../common/resources/storage/container.h"
#include "../../../common/resources/async_proxy.h"
#include "../../../common/utils/test/test.h"

#include "core_resources_intval.h"
#include "core_resources_sample.h"

/* Storage Test */
class core_resources_storage_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Resource Storage Test");

		// embedded storage test
		{
			core_data_buffer_t data;

			{
				auto p = std::make_shared<core_resources_embedded_sample>("string://10");

				core_resources_storage_container_t container;
				container.push_back(p);

				data = container.pack('SPC0');
			}

			{
				core_resources_storage_container_t container;

				container.unpack<core_resources_storage_base_t>(data, [](const std::string& type_name)
					{
						if (!core::is_type_name<core_resources_embedded_sample>(type_name))
							throw_exception(unknown_typename_exception, type_name);

						return std::make_shared<core_resources_embedded_sample>();
					});

				EXPECT_INT("Expect 1 element in container", container.size(), (size_t)1);

				std::string owner, name;
				std::shared_ptr<core_resources_storage_base_t> obj;

				std::tie(owner, name, obj) = container.front();

				auto p = std::static_pointer_cast<core_resources_embedded_sample>(obj);

				EXPECT_NOTNULL("Embedded storage object shall not be null", p);
				EXPECT_INT("Embedded storage resource value shall be 10", p->eval(), 10);
			}
		}

		// derived storage test
		{
			core_data_buffer_t data;

			{
				auto p = std::make_shared<core_resources_derived_sample>("string://10", 20);

				core_resources_storage_container_t container;
				container.push_back(p);

				data = container.pack('SPC0');
			}

			{
				core_resources_storage_container_t container;

				container.unpack<core_resources_storage_base_t>(data, [](const std::string& type_name)
					{
						if (!core::is_type_name<core_resources_derived_sample>(type_name))
							throw_exception(unknown_typename_exception, type_name);

						return std::make_shared<core_resources_derived_sample>();
					});

				EXPECT_INT("Expect 1 element in container", container.size(), (size_t)1);

				std::string owner, name;
				std::shared_ptr<core_resources_storage_base_t> obj;

				std::tie(owner, name, obj) = container.front();

				auto p = std::static_pointer_cast<core_resources_derived_sample>(obj);

				EXPECT_NOTNULL("Derived storage object shall not be null", p);
				EXPECT_INT("Derived storage resource value shall be 10", p->eval(), 10);
				EXPECT_INT("Derived storage other value shall be 20", p->other(), 20);
			}
		}
	}
};

REGISTER_TEST(core_resources_storage_test);