/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/data/hash/sha2.h"
#include "../../../common/data/buffer.h"
#include "../../../common/resources/storage/container.h"
#include "../../../common/resources/ext/buffer.h"
#include "../../../common/resources/ext/string.h"
#include "../../../common/utils/types/type.h"
#include "../../../common/utils/test/test.h"

#include "../../test-common/common.h"

/* sample module */
class core_storage_ext_mockup : public core_storage_module
{
public:
	IMPLEMENT_STORAGE;

	core_storage_ext_mockup(void) {}

	void set(int data)
	{
		this->m_string = "Hello World!";
		this->m_buffer = core::sha256(this->m_string);
	}

	const int get(void) const
	{
		EXPECT_STR("string shall be correct", this->m_string, "Hello World!");
		EXPECT_DATA("buffer shall be correct", this->m_buffer, core::sha256("Hello World!"));

		return -1;
	}

private:
	core_resources_ext_string_t m_string;
	core_resources_ext_buffer_t m_buffer;
};

BEGIN_STORAGE(core_storage_ext_mockup)
{
	DECLARE_STORAGE(core_storage_ext_mockup, m_string),
	DECLARE_STORAGE(core_storage_ext_mockup, m_buffer),
} END_STORAGE(core_storage_ext_mockup, core_storage_ext_mockup);

DECLARE_INSTANCEABLE_OBJECT(core_storage_ext_mockup, core_storage_module);

/* storage EXT tests */
class core_storage_ext_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Storage stdlib extension tests");

		// load/save tests
		{
			core_data_buffer_t buffer;

			// save
			{
				// create instance
				auto p = core::create_instance<core_storage_module>("core_storage_ext_mockup");

				// set value
				p->set(20);

				// storage object
				core_resources_storage_container_t container;
				container.push_back(p);

				buffer = container.pack('SPC0');
			}

			// reload
			{
				core_resources_storage_container_t container;

				container.unpack<core_storage_module>(buffer);

				EXPECT_INT("container shall have exactly 1 object", container.size(), (size_t)1);

				{
					std::shared_ptr<core_resources_storage_base_t> p;
					std::string owner, name;

					std::tie(owner, name, p) = container.front();

					auto obj = std::static_pointer_cast<core_storage_module>(p);

					EXPECT_NOTNULL("object pointer shall not be null", obj.get());

					int value = obj->get();

					EXPECT_INT("return value shall be -1", value, -1);
				}
			}
		}
	}
};

REGISTER_TEST(core_storage_ext_test);