/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/exception.h"
#include "../../../common/data/buffer.h"
#include "../../../common/resources/storage/container.h"
#include "../../../common/utils/test/test.h"

#include "../../test-common/common.h"

/* storage remote tests */
class core_storage_remote_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Storage remote tests");

		core_data_buffer_t buffer;

		// save
		{
			// create instance
			auto p = core::create_instance<core_storage_module>("core-test-lib/core_storage_remote_mockup");

			// set value
			p->set(10);

			EXPECT_INT("value shall be 10", p->get(), 10);

			// storage object
			core_resources_storage_container_t container;
			container.push_back(p);

			buffer = container.pack('SPC0');
		}

		// reload
		{
			core_resources_storage_container_t container;

			container.unpack<core_storage_module>(buffer);

			EXPECT_INT("container shall have exactly 1 object", container.size(), (size_t)1);

			{
				std::shared_ptr<core_resources_storage_base_t> p;
				std::string owner, name;

				std::tie(owner, name, p) = container.front();

				auto obj = std::static_pointer_cast<core_storage_module>(p);

				EXPECT_NOTNULL("object pointer shall not be null", obj.get());
				EXPECT_INT("object value shall be 10 after restore", obj->get(), 10);
			}
		}
	}
};

REGISTER_TEST(core_storage_remote_test);