/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/threads/jobs/base.h"
#include "../../../common/threads/jobs/manager.h"
#include "../../../common/threads/pool/manager.h"
#include "../../../common/utils/test/test.h"

/* job mockup class */
class core_threads_job_mockup : public core_threads_jobs_base_t
{
public:
	core_threads_job_mockup(void)
	{
		this->m_data = -1;
	}

	const int get(void) const
	{
		return this->m_data;
	}

private:
	virtual void _init(void) override
	{
		this->m_data = 0;
	}

	virtual void _run(void) override
	{
		this->m_data = 1;
	}

private:
	int m_data;
};

/* job test */
class core_threads_jobs_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Job tests");

		// start jobs manager
		core::get_instance<core_threads_jobs_manager_t>()->start();

		// test asynchronous single job
		{
			auto job = std::make_shared<core_threads_job_mockup>();

			EXPECT_NOTNULL("job shall not be null", job.get());
			EXPECT_INT("initial value shall be -1", job->get(), -1);

			EXPECT_TRUE("job shall not timeout", core::wait_for({ job }, 0.1));

			EXPECT_INT("asynchronous final value shall be 1", job->get(), 1);
		}

		// stop jobs manager
		core::get_instance<core_threads_jobs_manager_t>()->stop();

		// test synchronous single job
		{
			auto job = std::make_shared<core_threads_job_mockup>();

			EXPECT_NOTNULL("job shall not be null", job.get());
			EXPECT_INT("initial value shall be -1", job->get(), -1);

			EXPECT_TRUE("job shall not timeout", core::wait_for({ job }, 0.1));

			EXPECT_INT("synchronous final value shall be 1", job->get(), 1);
		}
	}
};

REGISTER_TEST(core_threads_jobs_test);