/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/threads/lock.h"
#include "../../../common/utils/test/test.h"

/* lock test */
class core_threads_lock_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Lock tests");

		// standard mutex
		{
			std::mutex lock;

			// lockable at first
			{
				EXPECT_TRUE("standard lock shall be lockable", lock.try_lock());

				lock.unlock();
			}

			// not lockable after lock guard
			{
				AUTOLOCK(lock);

				EXPECT_FALSE("standard lock shall not be lockable after lock", lock.try_lock());
			}

			// relockable after lock guard
			{
				EXPECT_TRUE("standard lock shall be lockable again", lock.try_lock());

				lock.unlock();
			}
		}

		// recursive mutex
		{
			std::recursive_mutex lock;

			// lockable at first
			{
				EXPECT_TRUE("recursive lock shall be lockable", lock.try_lock());

				lock.unlock();
			}

			// not lockable after lock guard
			{
				AUTOLOCK_EX(lock, std::recursive_mutex);

				EXPECT_TRUE("recursive lock shall still be lockable after lock", lock.try_lock());
			}

			// relockable after lock guard
			{
				EXPECT_TRUE("recursive lock shall be lockable again", lock.try_lock());

				lock.unlock();
			}
		}
	}
};

REGISTER_TEST(core_threads_lock_test);