/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/threads/base.h"
#include "../../../common/threads/event/flag.h"
#include "../../../common/utils/test/test.h"

/* thread mockup class */
class core_thread_mockup : public core_threads_base_t
{
public:

	// to prevent an eventual pure virtual function call
	~core_thread_mockup(void)
	{
		NOTHROW(stop());
	}

private:
	virtual void run(void) override
	{
		sleep(1e-3);
	}
};

/* simple action thread */
class core_thread_simple_action_mockup : public core_threads_base_t
{
public:
	// to prevent an eventual pure virtual function call
	~core_thread_simple_action_mockup(void)
	{
		NOTHROW(stop());
	}

	bool test(void) const
	{
		return !this->m_event.is_pending();
	}

private:
	virtual void run(void) override
	{
		this->m_event.trigger();
	}

	// quit condition
	virtual bool stop_condition(void) const override
	{
		return !this->m_event.is_pending();
	}

private:
	core_threads_event_flag_t m_event;
};

/* threads test */
class core_threads_thread_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Threads tests");

		// start & stop
		{
			core_thread_mockup m;

			EXPECT_FALSE("initial state shall be not running", m.is_running());

			m.start();

			EXPECT_TRUE("state shall now be running", m.is_running());

			m.stop();

			EXPECT_FALSE("state shall now be not running", m.is_running());
		}
		
		// simple action
		{
			core_thread_simple_action_mockup m;

			EXPECT_FALSE("initial state shall be pending", m.test());

			m.start();

			EXPECT_TRUE("thread shall not fail on wait", m.wait(0.1));
			EXPECT_TRUE("state shall now be completed", m.test());
		}

		// thread shall stop on dtor
		{
			core_thread_mockup m;

			m.start();
		}

		// start called twice
		{
			core_thread_mockup m;

			EXPECT_TRUE("thread shall be started", m.start());

			// should have no effect
			EXPECT_FALSE("thread shall not be started a second time", m.start());
		}
	}
};

REGISTER_TEST(core_threads_thread_test);