/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of he MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 * 
 * This Source Code is part of the More-CPP CORE library.
 * https://www.more-cpp.com
 */
#include "../../../common/utils/utils.h"
#include "../../../common/debug/exdebug.h"
#include "../../../common/exception.h"
#include "../../../common/utils/types/type.h"
#include "../../../common/utils/test/test.h"

class core_utils_type_mockup {};
class core_utils_unknown_type_mockup {};

REGISTER_TYPENAME_EX(core_utils_type_mockup, "core_utils_type_mockup", core_utils_type_mockup_t);

/* type test */
class core_utils_type_test : public core_utils_test_base_t
{
public:
	virtual void run(void) const override
	{
		_comment("Types tests");

		// name from type
		{
			EXPECT_STR("name from type shall be correct", core::get_name_from_type<core_utils_type_mockup>(), "core_utils_type_mockup");

			core_utils_type_mockup obj;

			EXPECT_STR("name from type shall be correct (pointer version)", core::get_name_from_type(&obj), "core_utils_type_mockup");
		}

		// unknown type
		EXPECT_EXCEPTION("name from type shall trigger exception for unknown types", [](void)
			{
				auto tmp = core::get_name_from_type<core_utils_unknown_type_mockup>();
			});

		// split_typename
		{
			auto s = core::split_typename("lib/type");

			EXPECT_STR("split_typename library name shall be \"lib\"", s.lib, "lib");
			EXPECT_STR("split_typename type name shall be \"type\"", s.type, "type");

			EXPECT_EXCEPTION("ill formated typename shall trigger exception", [](void)
				{
					auto s = core::split_typename("lib/type/wrong");
				});
		}

		// decorate_typename
		EXPECT_NOEXCEPTION("decorate_typename shall not trigger exception", [](void)
			{
				auto s = core::decorate_typename("type");
			});

		// get_type_from_name
		EXPECT_NOEXCEPTION("get_type_from_name shall not trigger exception", [](void)
			{
				auto s = core::get_type_from_name("core_utils_type_mockup");
			});

		// get_type_from_name
		EXPECT_EXCEPTION("get_type_from_name shall trigger exception for unknown names", [](void)
			{
				auto s = core::get_type_from_name("core_utils_type_unknown_mockup");
			});

		// decorate_typename
		EXPECT_EXCEPTION("ill formated typename shall trigger exception", [](void)
			{
				auto s = core::decorate_typename("lib/type");
			});

		// is_type_name
		{
			EXPECT_TRUE("is_type_name shall be true", core::is_type_name<core_utils_type_mockup>("core_utils_type_mockup"));
			EXPECT_FALSE("is_type_name shall be false", core::is_type_name<core_utils_type_mockup>("int"));
			EXPECT_TRUE("is_type_name shall be true", core::is_type_name("int", "int"));
			EXPECT_FALSE("is_type_name shall be false", core::is_type_name("int", "double"));
			EXPECT_TRUE("is_type_name shall be true", core::is_type_name("lib/int", "lib/int"));
			EXPECT_FALSE("is_type_name shall be false", core::is_type_name("lib/int", "int"));
		}
	}
};

REGISTER_TEST(core_utils_type_test);